@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /***************************************************************************
- * CVSID: $Id: nm-vpnc.c 4284 2008-11-14 17:04:29Z dcbw $
+ * CVSID: $Id: nm-vpnc.c 4366 2008-12-06 00:19:59Z dcbw $
  *
  * nm-vpnc.c : GNOME UI dialogs for configuring vpnc VPN connections
  *
@@ -308,7 +308,8 @@ static void
 init_one_pw_combo (VpncPluginUiWidget *self,
                    NMSettingVPN *s_vpn,
                    const char *combo_name,
-                   const char *key)
+                   const char *key,
+                   const char *entry_name)
 {
 	VpncPluginUiWidgetPrivate *priv = VPNC_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
 	int active = -1;
@@ -316,6 +317,19 @@ init_one_pw_combo (VpncPluginUiWidget *self,
 	GtkListStore *store;
 	GtkTreeIter iter;
 	const char *value = NULL;
+	guint32 default_idx = 1;
+
+	/* If there's already a password and the password type can't be found in
+	 * the VPN settings, default to saving it.  Otherwise, always ask for it.
+	 */
+	widget = glade_xml_get_widget (priv->xml, entry_name);
+	if (widget) {
+		const char *tmp;
+
+		tmp = gtk_entry_get_text (GTK_ENTRY (widget));
+		if (tmp && strlen (tmp))
+			default_idx = 0;
+	}
 
 	store = gtk_list_store_new (1, G_TYPE_STRING);
 	if (s_vpn)
@@ -346,7 +360,7 @@ init_one_pw_combo (VpncPluginUiWidget *self,
 	g_assert (widget);
 	gtk_combo_box_set_model (GTK_COMBO_BOX (widget), GTK_TREE_MODEL (store));
 	g_object_unref (store);
-	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), active < 0 ? 0 : active);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), active < 0 ? default_idx : active);
 	pw_type_changed_helper (self, widget);
 
 	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (pw_type_combo_changed_cb), self);
@@ -418,8 +432,15 @@ init_plugin_ui (VpncPluginUiWidget *self, NMConnection *connection, GError **err
 	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), active < 0 ? 0 : active);
 	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (stuff_changed_cb), self);
 
-	init_one_pw_combo (self, s_vpn, "user_pass_type_combo", NM_VPNC_KEY_XAUTH_PASSWORD_TYPE);
-	init_one_pw_combo (self, s_vpn, "group_pass_type_combo", NM_VPNC_KEY_SECRET_TYPE);
+	/* Fill the VPN passwords *before* initializing the PW type combos, since
+	 * knowing if there are passwords when initializing the combos is helpful.
+	 */
+	fill_vpn_passwords (self, connection);
+
+	init_one_pw_combo (self, s_vpn, "user_pass_type_combo",
+	                   NM_VPNC_KEY_XAUTH_PASSWORD_TYPE, "user_password_entry");
+	init_one_pw_combo (self, s_vpn, "group_pass_type_combo",
+	                   NM_VPNC_KEY_SECRET_TYPE, "group_password_entry");
 
 	widget = glade_xml_get_widget (priv->xml, "user_entry");
 	g_return_val_if_fail (widget != NULL, FALSE);
@@ -447,24 +468,31 @@ init_plugin_ui (VpncPluginUiWidget *self, NMConnection *connection, GError **err
 		natt_mode = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_NAT_TRAVERSAL_MODE);
 
 	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store, &iter, 0, _("Cisco UDP (default)"), 1, NM_VPNC_NATT_MODE_CISCO, -1);
+	gtk_list_store_set (store, &iter, 0, _("NAT-T when available (default)"), 1, NM_VPNC_NATT_MODE_NATT, -1);
 	if ((active < 0) && natt_mode) {
-		if (!strcmp (natt_mode, NM_VPNC_NATT_MODE_CISCO))
+		if (!strcmp (natt_mode, NM_VPNC_NATT_MODE_NATT))
 			active = 0;
 	}
 
 	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store, &iter, 0, _("NAT-T"), 1, NM_VPNC_NATT_MODE_NATT, -1);
+	gtk_list_store_set (store, &iter, 0, _("NAT-T always"), 1, NM_VPNC_NATT_MODE_NATT_ALWAYS, -1);
 	if ((active < 0) && natt_mode) {
-		if (!strcmp (natt_mode, NM_VPNC_NATT_MODE_NATT))
+		if (!strcmp (natt_mode, NM_VPNC_NATT_MODE_NATT_ALWAYS))
 			active = 1;
 	}
 
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter, 0, _("Cisco UDP"), 1, NM_VPNC_NATT_MODE_CISCO, -1);
+	if ((active < 0) && natt_mode) {
+		if (!strcmp (natt_mode, NM_VPNC_NATT_MODE_CISCO))
+			active = 2;
+	}
+
 	gtk_list_store_append (store, &iter);
 	gtk_list_store_set (store, &iter, 0, _("Disabled"), 1, NM_VPNC_NATT_MODE_NONE, -1);
 	if ((active < 0) && natt_mode) {
 		if (!strcmp (natt_mode, NM_VPNC_NATT_MODE_NONE))
-			active = 2;
+			active = 3;
 	}
 
 	widget = glade_xml_get_widget (priv->xml, "natt_combo");
@@ -493,8 +521,6 @@ init_plugin_ui (VpncPluginUiWidget *self, NMConnection *connection, GError **err
 	}
 	g_signal_connect (G_OBJECT (widget), "toggled", G_CALLBACK (stuff_changed_cb), self);
 
-	fill_vpn_passwords (self, connection);
-
 	widget = glade_xml_get_widget (priv->xml, "show_passwords_checkbutton");
 	g_return_val_if_fail (widget != NULL, FALSE);
 	g_signal_connect (G_OBJECT (widget), "toggled",
@@ -688,7 +714,7 @@ save_secrets (NMVpnPluginUiWidgetInterface *iface,
 	VpncPluginUiWidget *self = VPNC_PLUGIN_UI_WIDGET (iface);
 	VpncPluginUiWidgetPrivate *priv = VPNC_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
 	NMSettingConnection *s_con;
-	const char *str, *id, *uuid;
+	const char *id, *uuid;
 
 	s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
 	if (!s_con) {
@@ -955,11 +981,13 @@ import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 	NMSettingVPN *s_vpn;
 	GHashTable *pcf;
 	const char *buf;
-	gboolean have_value;
+	gboolean bool_value;
+	NMSettingIP4Config *s_ip4;
+	gint val;
 
 	pcf = pcf_file_load (path);
 	if (!pcf) {
-		g_set_error (error, 0, 0, "does not look like a %s VPN connection",
+		g_set_error (error, 0, 0, "does not look like a %s VPN connection (parse failed)",
 		             VPNC_PLUGIN_NAME);
 		return NULL;
 	}
@@ -972,18 +1000,11 @@ import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 	g_object_set (s_vpn, NM_SETTING_VPN_SERVICE_TYPE, NM_DBUS_SERVICE_VPNC, NULL);
 	nm_connection_add_setting (connection, NM_SETTING (s_vpn));
 
-	/* Connection name */
-	if ((buf = pcf_file_lookup_value (pcf, "main", "Description")))
-		g_object_set (s_con, NM_SETTING_CONNECTION_ID, buf, NULL);
-	else {
-		g_set_error (error, 0, 0, "does not look like a %s VPN connection (parse failed)",
-		             VPNC_PLUGIN_NAME);
-		g_object_unref (connection);
-		return NULL;
-	}
+	s_ip4 = NM_SETTING_IP4_CONFIG (nm_setting_ip4_config_new ());
+	nm_connection_add_setting (connection, NM_SETTING (s_ip4));
 
 	/* Gateway */
-	if ((buf = pcf_file_lookup_value (pcf, "main", "Host")))
+	if (pcf_file_lookup_string (pcf, "main", "Host", &buf))
 		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_GATEWAY, buf);
 	else {
 		g_set_error (error, 0, 0, "does not look like a %s VPN connection (no Host)",
@@ -993,7 +1014,7 @@ import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 	}
 
 	/* Group name */
-	if ((buf = pcf_file_lookup_value (pcf, "main", "GroupName")))
+	if (pcf_file_lookup_string (pcf, "main", "GroupName", &buf))
 		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_ID, buf);
 	else {
 		g_set_error (error, 0, 0, "does not look like a %s VPN connection (no GroupName)",
@@ -1004,25 +1025,29 @@ import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 
 	/* Optional settings */
 
-	buf = pcf_file_lookup_value (pcf, "main", "UserName");
-	have_value = buf == NULL ? FALSE : strlen (buf) > 0;
-	if (have_value)
+	/* Connection name */
+	if (pcf_file_lookup_string (pcf, "main", "Description", &buf))
+		g_object_set (s_con, NM_SETTING_CONNECTION_ID, buf, NULL);
+
+	if (pcf_file_lookup_string (pcf, "main", "UserName", &buf))
 		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_XAUTH_USER, buf);
 
-	buf = pcf_file_lookup_value (pcf, "main", "UserPassword");
-	have_value = buf == NULL ? FALSE : strlen (buf) > 0;
-	if (have_value)
+	if (pcf_file_lookup_string (pcf, "main", "UserPassword", &buf))
 		nm_setting_vpn_add_secret (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD, buf);
 
-	buf = pcf_file_lookup_value (pcf, "main", "GroupPwd");
-	have_value = buf == NULL ? FALSE : strlen (buf) > 0;
-	if (have_value)
+	if (pcf_file_lookup_bool (pcf, "main", "SaveUserPassword", &bool_value)) {
+		if (bool_value) {
+			nm_setting_vpn_add_data_item (s_vpn,
+			                              NM_VPNC_KEY_XAUTH_PASSWORD_TYPE,
+			                              NM_VPNC_PW_TYPE_SAVE);
+		}
+	}
+
+	if (pcf_file_lookup_string (pcf, "main", "GroupPwd", &buf))
 		nm_setting_vpn_add_secret (s_vpn, NM_VPNC_KEY_SECRET, buf);
 	else {
 		/* Handle encrypted passwords */
-		buf = pcf_file_lookup_value (pcf, "main", "enc_GroupPwd");
-		have_value = buf == NULL ? FALSE : strlen (buf) > 0;
-		if (have_value) {
+		if (pcf_file_lookup_string (pcf, "main", "enc_GroupPwd", &buf)) {
 			char *decrypted;
 
 			decrypted = decrypt_cisco_key (buf);
@@ -1034,57 +1059,85 @@ import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 		}
 	}
 
-	buf = pcf_file_lookup_value (pcf, "main", "NTDomain");
-	have_value = buf == NULL ? FALSE : strlen (buf) > 0;
-	if (have_value)
+	if (pcf_file_lookup_string (pcf, "main", "NTDomain", &buf))
 		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_DOMAIN, buf);
 
-	buf = pcf_file_lookup_value (pcf, "main", "SingleDES");
-	have_value = (buf == NULL ? FALSE : strcmp (buf, "0") != 0);
-	if (have_value)
-		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_SINGLE_DES, "yes");
+	if (pcf_file_lookup_bool (pcf, "main", "SingleDES", &bool_value)) {
+		if (bool_value)
+			nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_SINGLE_DES, "yes");
+	}
 
-	/* Default is enabled, only disabled if explicit EnableNat=0 exists */
-	buf = pcf_file_lookup_value (pcf, "main", "EnableNat");
-	have_value = (buf ? strncmp (buf, "0", 1) == 0 : FALSE);
-	if (have_value)
-		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_NAT_TRAVERSAL_MODE, NM_VPNC_NATT_MODE_NATT);
+	/* Disable all NAT Traversal if explicit EnableNat=0 exists, otherwise
+	 * default to NAT-T which is newer and standardized.  If EnableNat=1, then
+	 * use Cisco-UDP like always; but if the key "X-NM-Use-NAT-T" is set, then
+	 * use NAT-T.  If the key "X-NM-Force-NAT-T" is set then force NAT-T always
+	 * on.  See vpnc documentation for more information on what the different
+	 * NAT modes are.
+	 */
+	nm_setting_vpn_add_data_item (s_vpn,
+	                              NM_VPNC_KEY_NAT_TRAVERSAL_MODE,
+	                              NM_VPNC_NATT_MODE_CISCO);
 
-	if ((buf = pcf_file_lookup_value (pcf, "main", "PeerTimeout"))) {
-		long int val;
+	if (pcf_file_lookup_bool (pcf, "main", "EnableNat", &bool_value)) {
+		if (bool_value) {
+			gboolean natt = FALSE, force_natt = FALSE;
 
-		errno = 0;
-		val = strtol (buf, NULL, 10);
-		if ((errno == 0) && ((val == 0) || ((val >= 10) && (val <= 86400)))) {
+			if (!pcf_file_lookup_bool (pcf, "main", "X-NM-Use-NAT-T", &natt))
+				natt = FALSE;
+			if (!pcf_file_lookup_bool (pcf, "main", "X-NM-Force-NAT-T", &force_natt))
+				force_natt = FALSE;
+
+			/* force-natt takes precence over plain natt */
+			if (force_natt) {
+				nm_setting_vpn_add_data_item (s_vpn,
+				                              NM_VPNC_KEY_NAT_TRAVERSAL_MODE,
+				                              NM_VPNC_NATT_MODE_NATT_ALWAYS);
+			} else if (natt) {
+				nm_setting_vpn_add_data_item (s_vpn,
+				                              NM_VPNC_KEY_NAT_TRAVERSAL_MODE,
+				                              NM_VPNC_NATT_MODE_NATT);
+			}
+		} else {
+			nm_setting_vpn_add_data_item (s_vpn,
+			                              NM_VPNC_KEY_NAT_TRAVERSAL_MODE,
+			                              NM_VPNC_NATT_MODE_NONE);
+		}
+	}
+
+	if (pcf_file_lookup_int (pcf, "main", "PeerTimeout", &val)) {
+		if ((val == 0) || ((val >= 10) && (val <= 86400))) {
 			char *tmp = g_strdup_printf ("%d", (gint) val);
 			nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_DPD_IDLE_TIMEOUT, tmp);
 			g_free (tmp);
 		}
 	}
 
-	buf = pcf_file_lookup_value (pcf, "main", "X-NM-Routes");
-	have_value = buf == NULL ? FALSE : strlen (buf) > 0;
-	if (have_value) {
-		NMSettingIP4Config *s_ip4;
-
-		s_ip4 = NM_SETTING_IP4_CONFIG (nm_setting_ip4_config_new ());
-		nm_connection_add_setting (connection, NM_SETTING (s_ip4));
-		add_routes (s_ip4, buf);
+	if (pcf_file_lookup_bool (pcf, "main", "EnableLocalLAN", &bool_value)) {
+		if (bool_value)
+			g_object_set (s_ip4, NM_SETTING_IP4_CONFIG_NEVER_DEFAULT, TRUE, NULL);
 	}
 
-	if ((buf = pcf_file_lookup_value (pcf, "main", "TunnelingMode"))) {
+	if (pcf_file_lookup_string (pcf, "main", "DHGroup", &buf)) {
+		if (!strcmp (buf, "1") || !strcmp (buf, "2") || !strcmp (buf, "5"))
+			nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_DHGROUP, buf);
+	}
+
+	if (pcf_file_lookup_string (pcf, "main", "X-NM-Routes", &buf))
+		add_routes (s_ip4, buf);
+
+	if (pcf_file_lookup_int (pcf, "main", "TunnelingMode", &val)) {
 		/* If applicable, put up warning that TCP tunneling will be disabled */
 
-		if (strncmp (buf, "1", 1) == 0) {
+		if (val == 1) {
 			GtkWidget *dialog;
 			char *basename;
 
 			basename = g_path_get_basename (path);
 			dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
-											 GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
-											 _("TCP tunneling not supported"));
+			                                 GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
+			                                 _("TCP tunneling not supported"));
 			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-													  _("The VPN settings file '%s' specifies that VPN traffic should be tunneled through TCP which is currently not supported in the vpnc software.\n\nThe connection can still be created, with TCP tunneling disabled, however it may not work as expected."), basename);
+			                                          _("The VPN settings file '%s' specifies that VPN traffic should be tunneled through TCP which is currently not supported in the vpnc software.\n\nThe connection can still be created, with TCP tunneling disabled, however it may not work as expected."), basename);
 			g_free (basename);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
@@ -1114,8 +1167,13 @@ export (NMVpnPluginUiInterface *iface,
 	const char *username = NULL;
 	const char *domain = NULL;
 	const char *peertimeout = NULL;
+	const char *dhgroup = NULL;
 	GString *routes = NULL;
 	gboolean success = FALSE;
+	guint32 routes_count = 0;
+	gboolean save_password = FALSE;
+	gboolean use_natt = FALSE;
+	gboolean use_force_natt = FALSE;
 
 	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
 	s_ip4 = (NMSettingIP4Config *) nm_connection_get_setting (connection, NM_TYPE_SETTING_IP4_CONFIG);
@@ -1150,21 +1208,42 @@ export (NMVpnPluginUiInterface *iface,
 
 	value = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_DOMAIN);
 	if (value && strlen (value))
-		domain =  value;
+		domain = value;
 
 	value = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_SINGLE_DES);
 	if (value && !strcmp (value, "yes"))
 		singledes = TRUE;
 
 	value = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_NAT_TRAVERSAL_MODE);
-	if (value && strlen (value) && strcmp (value, NM_VPNC_NATT_MODE_NONE))
-		enablenat = TRUE;
+	if (value && strlen (value)) {
+		if (!strcmp (value, NM_VPNC_NATT_MODE_CISCO)) {
+			enablenat = TRUE;
+			use_natt = FALSE;
+		} else if (!strcmp (value, NM_VPNC_NATT_MODE_NATT)) {
+			enablenat = TRUE;
+			use_natt = TRUE;
+		} else if (!strcmp (value, NM_VPNC_NATT_MODE_NATT_ALWAYS)) {
+			enablenat = TRUE;
+			use_natt = TRUE;
+			use_force_natt = TRUE;
+		}
+	}
 
 	value = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_DPD_IDLE_TIMEOUT);
 	if (value && strlen (value))
 		peertimeout = value;
 
-	routes = g_string_new ("");
+	value = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_DHGROUP);
+	if (value && strlen (value))
+		dhgroup = value;
+
+	value = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD_TYPE);
+	if (value && strlen (value)) {
+		if (!strcmp (value, NM_VPNC_PW_TYPE_SAVE))
+			save_password = TRUE;
+	}
+
+	routes = g_string_new ("X-NM-Routes=");
 	if (s_ip4 && nm_setting_ip4_config_get_num_routes (s_ip4)) {
 		int i;
 
@@ -1173,14 +1252,20 @@ export (NMVpnPluginUiInterface *iface,
 			char str_addr[INET_ADDRSTRLEN + 1];
 			struct in_addr num_addr;
 
-			if (routes->len)
+			if (routes_count)
 				g_string_append_c (routes, ' ');
 
 			num_addr.s_addr = nm_ip4_route_get_dest (route);
 			if (inet_ntop (AF_INET, &num_addr, &str_addr[0], INET_ADDRSTRLEN + 1))
 				g_string_append_printf (routes, "%s/%d", str_addr, nm_ip4_route_get_prefix (route));
+
+			routes_count++;
 		}
 	}
+	if (!routes_count) {
+		g_string_free (routes, TRUE);
+		routes = NULL;
+	}
 
 	fprintf (f, 
 		 "[main]\n"
@@ -1194,7 +1279,7 @@ export (NMVpnPluginUiInterface *iface,
 		 "ISPConnect=\n"
 		 "ISPCommand=\n"
 		 "Username=%s\n"
-		 "SaveUserPassword=0\n"
+		 "SaveUserPassword=%s\n"
 		 "EnableBackup=0\n"
 		 "BackupServer=\n"
 		 "EnableNat=%s\n"
@@ -1203,7 +1288,7 @@ export (NMVpnPluginUiInterface *iface,
 		 "CertPath=\n"
 		 "CertSubjectName=\n"
 		 "CertSerialHash=\n"
-		 "DHGroup=2\n"
+		 "DHGroup=%s\n"
 		 "ForceKeepAlives=0\n"
 		 "enc_GroupPwd=\n"
 		 "UserPassword=\n"
@@ -1220,16 +1305,22 @@ export (NMVpnPluginUiInterface *iface,
 		 "EnableSplitDNS=1\n"
 		 "SingleDES=%s\n"
 		 "SPPhonebook=\n"
-		 "%s",
-		 /* Description */ nm_setting_connection_get_id (s_con),
-		 /* Host */        gateway,
-		 /* GroupName */   groupname,
-		 /* Username */    username != NULL ? username : "",
-		 /* EnableNat */   enablenat ? "1" : "0",
-		 /* NTDomain */    domain != NULL ? domain : "",
-		 /* PeerTimeout */ peertimeout != NULL ? peertimeout : "0",
-		 /* SingleDES */   singledes ? "1" : "0",
-		 /* X-NM-Routes */ routes->str ? routes->str : "");
+		 "X-NM-Use-NAT-T=%s\n"
+		 "X-NM-Force-NAT-T=%s\n"
+		 "%s\n",
+		 /* Description */   nm_setting_connection_get_id (s_con),
+		 /* Host */          gateway,
+		 /* GroupName */     groupname,
+		 /* Username */      username != NULL ? username : "",
+		 /* Save Password */ save_password ? "1" : "0",
+		 /* EnableNat */     enablenat ? "1" : "0",
+		 /* DHGroup */       dhgroup != NULL ? dhgroup : "2",
+		 /* NTDomain */      domain != NULL ? domain : "",
+		 /* PeerTimeout */   peertimeout != NULL ? peertimeout : "0",
+		 /* SingleDES */     singledes ? "1" : "0",
+		 /* X-NM-Use-NAT-T */ use_natt ? "1" : "0",
+		 /* X-NM-Force-NAT-T */ use_force_natt ? "1" : "0",
+		 /* X-NM-Routes */   (routes && routes->str) ? routes->str : "");
 
 	success = TRUE;
 
