@@ -47,6 +47,7 @@
 #define NM_VPNC_KEY_DOMAIN "Domain"
 #define NM_VPNC_KEY_DHGROUP "IKE DH Group"
 #define NM_VPNC_KEY_PERFECT_FORWARD "Perfect Forward Secrecy"
+#define NM_VPNC_KEY_VENDOR "Vendor"
 #define NM_VPNC_KEY_APP_VERSION "Application Version"
 #define NM_VPNC_KEY_SINGLE_DES "Enable Single DES"
 #define NM_VPNC_KEY_NO_ENCRYPTION "Enable no encryption"
@@ -54,9 +55,10 @@
 #define NM_VPNC_KEY_DPD_IDLE_TIMEOUT "DPD idle timeout (our side)"
 #define NM_VPNC_KEY_CISCO_UDP_ENCAPS_PORT "Cisco UDP Encapsulation Port"
 
-#define NM_VPNC_NATT_MODE_NATT "natt"
-#define NM_VPNC_NATT_MODE_NONE "none"
-#define NM_VPNC_NATT_MODE_CISCO "cisco-udp"
+#define NM_VPNC_NATT_MODE_NATT        "natt"
+#define NM_VPNC_NATT_MODE_NONE        "none"
+#define NM_VPNC_NATT_MODE_NATT_ALWAYS "force-natt"
+#define NM_VPNC_NATT_MODE_CISCO       "cisco-udp"
 
 #define NM_VPNC_PW_TYPE_SAVE   "save"
 #define NM_VPNC_PW_TYPE_ASK    "ask"
